const { ServerStyleSheet } = require('styled-components')
const styleSheet = require('styled-components/lib/models/StyleSheet')

const STYLE_TAGS_REGEXP = /<style[^>]*>([^<]*)</g

function isOverV2() {
  return Boolean(ServerStyleSheet)
}

function isServer() {
  return typeof document === 'undefined'
}

function parseCSSfromHTML(html) {
  let css = ''
  let matches
  while ((matches = STYLE_TAGS_REGEXP.exec(html)) !== null) {
    css += matches[1].trim()
  }
  return css
}

function getCSS() {
  const overV2 = isOverV2()
  if (overV2 && isServer()) {
    return parseCSSfromHTML(new ServerStyleSheet().getStyleTags())
  } else if (overV2) {
    return parseCSSfromHTML(styleSheet.default.instance.toHTML())
  }
  return styleSheet.rules().map(rule => rule.cssText).join('\n')
}

function getClassNames(node) {
  const classNames = []
  if (node.children) {
    node.children.slice().reverse().forEach(child => (
      Array.prototype.unshift.apply(classNames, getClassNames(child))
    ))
  }
  if (node.props && node.props.className) {
    Array.prototype.unshift.apply(
      classNames,
      node.props.className.split(/\s/)
    )
  }
  return classNames
}

function hasRule(classNames, selectors) {
  return classNames.some(
    className => selectors.some(selector => selector.indexOf(className) > -1)
  )
}

module.exports = {
  isOverV2,
  isServer,
  parseCSSfromHTML,
  getCSS,
  getClassNames,
  hasRule,
}

